import { assertEquals } from '@std/assert'
import { describe, it } from '@std/testing/bdd'
import { returnsNext, stub } from '@std/testing/mock'
import { fetchEvents } from './fetch_events.ts'
import {
  aPageResponse,
  emptyPageResponse,
  OnlinePermanentEvent,
  OnlineTimedEvent,
  RealPermanentEvent,
  RealTimedEvent,
} from './providers/jasd/tests/fixtures.ts'

export type ResponsePayload = Record<string, unknown> | Error

export const fakeFetch = (response: ResponsePayload) => {
  return stub(
    globalThis,
    'fetch',
    returnsNext([Promise.resolve(new Response(JSON.stringify(response)))]),
  )
}

describe('fetchEvents', () => {
  it('can make sense of an empty response', async () => {
    using _ = fakeFetch(emptyPageResponse)

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result, {
      data: [],
      totalResults: 0,
    })
  })

  it('skips permanent events without start and end date', async () => {
    using _ = fakeFetch(aPageResponse([RealPermanentEvent], true, true))

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result, {
      data: [],
      totalResults: 0,
    })
  })

  it('can fetch the only page of 1 event', async () => {
    using _ = fakeFetch(aPageResponse([RealTimedEvent], true, true))

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result, {
      data: [
        {
          id: '4610',
          title: 'RealTimedEvent',
          organisationName: 'Allerweltskino e.V.',
          startDate: '2025-03-26T00:00:00+01:00',
          endDate: '2025-03-26T00:00:00+01:00',
          location: {
            city: 'Köln',
            street: 'Bahnhofstr.',
            streetNo: '41',
            zipCode: '50674',
          },
          isRemote: false,
          imageUrl: 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files/activities/act5.jpg',
          eventDetails: {
            description:
              '<p>Jeden Dienstag (bis auf eine Sommer- und Winterpause) zeigt der Verein Allerweltskino e.V. im Kölner Filmtheater Off Broadway einen Nachhaltigkeitsfilm. Das Programm ist im Internet abrufbar (siehe Kasten rechts).</p>',
            externalLink: 'https://www.allerweltskino.de',
            meetingLink: 'https://www.allerweltskino.de',
          },
        },
      ],
      totalResults: 1,
    })
  })

  it('filters timed events from different types of events', async () => {
    using _ = fakeFetch(aPageResponse(
      [
        RealTimedEvent,
        RealPermanentEvent,
        OnlineTimedEvent,
        OnlinePermanentEvent,
      ],
      true,
      true,
    ))

    const result = await fetchEvents({ offset: 0, limit: 5 })

    assertEquals(result.totalResults, 2, 'returns only two timed events')
    assertEquals(result.data[0].title, 'RealTimedEvent')
    assertEquals(result.data[1].title, 'OnlineTimedEvent')
  })

  it('keeps fetching pages until satisfying the requested limit of 1', async () => {
    //be aware the current implementation filters permanent events, so this is more of a hypothetical scenario
    const eventsFirstPage = [
      OnlinePermanentEvent,
      OnlinePermanentEvent,
    ]
    const eventsSecondPage = [
      OnlineTimedEvent,
      OnlinePermanentEvent,
    ]
    const eventsThirdPage = [
      OnlinePermanentEvent,
      OnlinePermanentEvent,
    ]

    const firsPageResponse = Promise.resolve(
      new Response(JSON.stringify(aPageResponse([eventsFirstPage], true, false))),
    )
    const secondPageResponse = Promise.resolve(
      new Response(JSON.stringify(aPageResponse([eventsSecondPage], false, false))),
    )
    const thirdPageResponse = Promise.resolve(
      new Response(JSON.stringify(aPageResponse([eventsThirdPage], false, true))),
    )

    using _ = stub(
      globalThis,
      'fetch',
      returnsNext([firsPageResponse, secondPageResponse, thirdPageResponse]),
    )

    const result = await fetchEvents({ offset: 0, limit: 1 })

    assertEquals(result.totalResults, 1, 'returns a single event from the second fetched page')
    assertEquals(result.data[0].title, 'OnlineTimedEvent', 'its the OnlineTimedEvent')
    assertEquals(_.calls.length, 2, 'only fetches two pages')
  })
})
